import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'CameraKitController.dart';

enum CameraFlashMode { on, off, auto }

class CameraKitView extends StatefulWidget {
  Function onBarcodeRead;
  Function onTakePicture;

  bool hasBarcodeReader;
  bool hasTakePicture;
  CameraFlashMode previewFlashMode;

  _BarcodeScannerViewState viewState;

  CameraKitController cameraKitController;

  CameraKitView({Key key,
    this.hasBarcodeReader = false,
    this.onBarcodeRead,
    this.hasTakePicture = false,
    this.onTakePicture,
    this.previewFlashMode = CameraFlashMode.auto,
    this.cameraKitController})
      : super(key: key) {
    cameraKitController.setView(this);

  }

  dispose(){
    viewState.dispose();
  }

  @override
  State<StatefulWidget> createState() {
    viewState = _BarcodeScannerViewState();
    return viewState;
  }
}

class _BarcodeScannerViewState extends State<CameraKitView>
    with WidgetsBindingObserver {
  NativeCameraKitController controller;

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addObserver(this);
  }

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.android) {
      return  AndroidView(
        viewType: 'plugins/camera_kit',
        onPlatformViewCreated: _onPlatformViewCreated,
      );
    } else {
    return UiKitView(
    viewType: 'plugins/camera_kit',
    onPlatformViewCreated: _onPlatformViewCreated,
    );
    }
    return Text(
    '$defaultTargetPlatform is not yet supported by the text_view plugin');
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    switch (state) {
      case AppLifecycleState.resumed:
        print("Flutter Life Cycle: resumed");
        controller.resumeCamera();
        break;
      case AppLifecycleState.inactive:
        print("Flutter Life Cycle: inactive");

        break;
      case AppLifecycleState.paused:
        print("Flutter Life Cycle: paused");
        controller.pauseCamera();
        break;
      default:
        break;
    }
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }

  void _onPlatformViewCreated(int id) {
    this.controller = new NativeCameraKitController._(id, context, widget);
    this.controller.initCamera();
  }
}

class NativeCameraKitController {
  BuildContext context;
  CameraKitView widget;

  NativeCameraKitController._(int id, this.context, this.widget)
      : _channel = new MethodChannel('plugins/camera_kit');

  final MethodChannel _channel;

  Future<dynamic> nativeMethodCallHandler(MethodCall methodCall) async {
    print("Flutter barcode read: " + methodCall.method);
    if (methodCall.method == "onBarcodeRead") {
      if (widget.onBarcodeRead != null)
        widget.onBarcodeRead(methodCall.arguments);
    } else if (methodCall.method == "onTakePicture") {
      if (widget.onTakePicture != null)
        widget.onTakePicture(methodCall.arguments);
    }

    return null;
  }

  String _getCharFlashMode(CameraFlashMode cameraFlashMode) {
    String flashMode;
    switch (cameraFlashMode) {
      case CameraFlashMode.auto:
        flashMode = "A";
        break;
      case CameraFlashMode.on:
        flashMode = "O";
        break;
      case CameraFlashMode.off:
        flashMode = "F";
        break;
    }
    return flashMode;
  }

  void initCamera() async {
    _channel.setMethodCallHandler(nativeMethodCallHandler);
    _channel.invokeMethod('requestPermission').then((value) {
      if (value) {
        _channel.invokeMethod('initCamera', {
          "hasTakePicture": widget.hasTakePicture,
          "hasBarcodeReader": widget.hasBarcodeReader,
          "flashMode": _getCharFlashMode(widget.previewFlashMode)
        });
      }
    });
  }


  Future<void> resumeCamera() async {
    return _channel.invokeMethod('resumeCamera');
  }

  Future<void> pauseCamera() async {
    return _channel.invokeMethod('pauseCamera');
  }

  Future<void> closeCamera() {
    return _channel.invokeMethod('closeCamera');
  }

  Future<String> takePicture(CameraFlashMode captureFlashMode) async {
    if (captureFlashMode == null)
      captureFlashMode = widget.previewFlashMode;
    return _channel.invokeMethod(
        'takePicture', {"flashMode": _getCharFlashMode(captureFlashMode)});
  }

  Future<void> changeFlashMode(CameraFlashMode captureFlashMode) {
    return _channel.invokeMethod(
        'changeFlashMode', {"flashMode": _getCharFlashMode(captureFlashMode)});
  }


}
